@@ -1,4 +1,4 @@
-#![doc = include_str!("../README.md")]
+#![doc = include_str ! ("../README.md")]
 
 pub mod artifacts;
 
@@ -8,9 +8,11 @@ use std::collections::btree_map::Entry;
 pub mod cache;
 
 mod compile;
+
 pub use compile::*;
 
 mod config;
+
 pub use config::{
     AllowedLibPaths, Artifact, ArtifactOutput, MinimalCombinedArtifacts, ProjectPathsConfig,
     SolcConfig,
@@ -22,6 +24,7 @@ use crate::{artifacts::Source, cache::SolFilesCache};
 
 pub mod error;
 pub mod utils;
+
 use crate::artifacts::Sources;
 use error::Result;
 use std::{
@@ -87,15 +90,18 @@ impl Project {
 }
 
 impl<Artifacts: ArtifactOutput> Project<Artifacts> {
+    #[tracing::instrument(skip_all, name = "Project::write_cache_file")]
     fn write_cache_file(
         &self,
         sources: Sources,
         artifacts: Vec<(PathBuf, Vec<String>)>,
     ) -> Result<()> {
+        tracing::trace!("inserting files to cache");
         let mut cache = SolFilesCache::builder()
             .root(&self.paths.root)
             .solc_config(self.solc_config.clone())
-            .insert_files(sources)?;
+            .insert_files(sources, Some(self.paths.cache.clone()))?;
+        tracing::trace!("files inserted");
 
         // add the artifacts for each file to the cache entry
         for (file, artifacts) in artifacts {
@@ -107,10 +113,13 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
         if let Some(cache_dir) = self.paths.cache.parent() {
             fs::create_dir_all(cache_dir)?
         }
-        cache.write(&self.paths.cache)
+        cache.write(&self.paths.cache)?;
+
+        Ok(())
     }
 
     /// Returns all sources found under the project's sources path
+    #[tracing::instrument(skip_all, fields(name = "sources"))]
     pub fn sources(&self) -> io::Result<Sources> {
         Source::read_all_from(self.paths.sources.as_path())
     }
@@ -164,11 +173,15 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
 
     /// NB: If the `svm` feature is enabled, this function will automatically detect
     /// solc versions across files.
+    #[tracing::instrument(skip_all, name = "compile")]
     pub fn compile(&self) -> Result<ProjectCompileOutput<Artifacts>> {
+        tracing::trace!("sources");
         let sources = self.sources()?;
+        tracing::trace!("done");
 
         #[cfg(all(feature = "svm", feature = "async"))]
         if self.auto_detect {
+            tracing::trace!("auto-compile");
             return self.svm_compile(sources)
         }
 
@@ -180,12 +193,20 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
     }
 
     #[cfg(all(feature = "svm", feature = "async"))]
+    #[tracing::instrument(skip(self, sources))]
     fn svm_compile(&self, sources: Sources) -> Result<ProjectCompileOutput<Artifacts>> {
         // split them by version
         let mut sources_by_version = BTreeMap::new();
+        // we store the solc versions by path, in case there exists a corrupt solc binary
+        let mut solc_versions = HashMap::new();
+
+        // TODO: Rayon
+        // tracing::trace!("parsing sources");
         for (path, source) in sources.into_iter() {
             // will detect and install the solc version
+            // tracing::trace!("finding version {}", path.display());
             let version = Solc::detect_version(&source)?;
+            // tracing::trace!("found {}", version);
             // gets the solc binary for that version, it is expected tha this will succeed
             // AND find the solc since it was installed right above
             let mut solc = Solc::find_svm_installed_version(version.to_string())?
@@ -194,26 +215,36 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
             if !self.allowed_lib_paths.0.is_empty() {
                 solc = solc.arg("--allow-paths").arg(self.allowed_lib_paths.to_string());
             }
+            solc_versions.insert(solc.solc.clone(), version);
             let entry = sources_by_version.entry(solc).or_insert_with(BTreeMap::new);
-            entry.insert(path, source);
+            entry.insert(path.clone(), source);
         }
+        // tracing::trace!("done");
 
         let mut compiled =
             ProjectCompileOutput::with_ignored_errors(self.ignored_error_codes.clone());
 
         // run the compilation step for each version
+        tracing::trace!("compiling sources with viable solc versions");
         for (solc, sources) in sources_by_version {
+            let span = tracing::trace_span!("solc", "{}", solc.version_short()?);
+            let _enter = span.enter();
+
+            // verify that this solc version's checksum matches the checksum found remotely. If
+            // not, re-install the same version.
+            let version = solc_versions.get(&solc.solc).unwrap();
+            if let Err(_e) = solc.verify_checksum() {
+                tracing::trace!("corrupted solc version, redownloading...");
+                Solc::blocking_install(version)?;
+                tracing::trace!("done.");
+            }
+            // once matched, proceed to compile with it
+            tracing::trace!("compiling_with_version");
             compiled.extend(self.compile_with_version(&solc, sources)?);
+            tracing::trace!("done compiling_with_version");
         }
-        if !compiled.has_compiled_contracts() &&
-            !compiled.has_compiler_errors() &&
-            self.cached &&
-            self.paths.cache.exists()
-        {
-            let cache = SolFilesCache::read(&self.paths.cache)?;
-            let artifacts = cache.read_artifacts::<Artifacts>(&self.paths.artifacts)?;
-            compiled.artifacts.extend(artifacts);
-        }
+        tracing::trace!("compiled sources with viable solc versions");
+
         Ok(compiled)
     }
 
@@ -222,6 +253,8 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
         solc: &Solc,
         mut sources: Sources,
     ) -> Result<ProjectCompileOutput<Artifacts>> {
+        let span = tracing::trace_span!("compiling");
+        let _enter = span.enter();
         // add all libraries to the source set while keeping track of their actual disk path
         // (`contracts/contract.sol` -> `/Users/.../contracts.sol`)
         let mut source_name_to_path = HashMap::new();
@@ -229,30 +262,46 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
         // `contracts/contract.sol`)
         let mut path_to_source_name = HashMap::new();
 
+        tracing::trace!("resolving libraries");
         for (import, (source, path)) in self.resolved_libraries(&sources)? {
             // inserting with absolute path here and keep track of the source name <-> path mappings
             sources.insert(path.clone(), source);
             path_to_source_name.insert(path.clone(), import.clone());
             source_name_to_path.insert(import, path);
         }
+        tracing::trace!("resolved libraries");
 
         // If there's a cache set, filter to only re-compile the files which were changed
-        let sources = if self.cached && self.paths.cache.exists() {
-            let cache = SolFilesCache::read(&self.paths.cache)?;
+        let (sources, cached_artifacts) = if self.cached && self.paths.cache.exists() {
+            tracing::trace!("reading solfiles cache for incremental compilation");
+            let mut cache = SolFilesCache::read(&self.paths.cache)?;
+            cache.remove_missing_files();
+            tracing::trace!("done reading solfiles cache for incremental compilation");
             let changed_files = cache.get_changed_or_missing_artifacts_files::<Artifacts>(
                 sources,
                 Some(&self.solc_config),
                 &self.paths.artifacts,
             );
+            cache.remove_changed_files(&changed_files);
+
+            let cached_artifacts = if self.paths.artifacts.exists() {
+                tracing::trace!("reading artifacts from cache..");
+                let artifacts = cache.read_artifacts::<Artifacts>(&self.paths.artifacts)?;
+                tracing::trace!("done reading artifacts from cache");
+                artifacts
+            } else {
+                BTreeMap::default()
+            };
 
             // if nothing changed and all artifacts still exist
             if changed_files.is_empty() {
-                let artifacts = cache.read_artifacts::<Artifacts>(&self.paths.artifacts)?;
-                return Ok(ProjectCompileOutput::from_unchanged(artifacts))
+                tracing::trace!("no change");
+                return Ok(ProjectCompileOutput::from_unchanged(cached_artifacts))
             }
-            changed_files
+            // There are changed files and maybe some cached files
+            (changed_files, cached_artifacts)
         } else {
-            sources
+            (sources, BTreeMap::default())
         };
 
         // replace absolute path with source name to make solc happy
@@ -261,7 +310,9 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
         let input = CompilerInput::with_sources(sources)
             .normalize_evm_version(&solc.version()?)
             .with_remappings(self.paths.remappings.clone());
+        tracing::trace!("calling solc");
         let output = solc.compile(&input)?;
+        tracing::trace!("compiled input, output has error: {}", output.has_error());
         if output.has_error() {
             return Ok(ProjectCompileOutput::from_compiler_output(
                 output,
@@ -292,7 +343,12 @@ impl<Artifacts: ArtifactOutput> Project<Artifacts> {
         if !self.no_artifacts {
             Artifacts::on_output(&output, &self.paths)?;
         }
-        Ok(ProjectCompileOutput::from_compiler_output(output, self.ignored_error_codes.clone()))
+
+        Ok(ProjectCompileOutput::from_compiler_output_and_cache(
+            output,
+            cached_artifacts,
+            self.ignored_error_codes.clone(),
+        ))
     }
 }
 
@@ -501,6 +557,14 @@ impl<T: ArtifactOutput> ProjectCompileOutput<T> {
         }
     }
 
+    pub fn from_compiler_output_and_cache(
+        compiler_output: CompilerOutput,
+        cache: BTreeMap<PathBuf, T::Artifact>,
+        ignored_error_codes: Vec<u64>,
+    ) -> Self {
+        Self { compiler_output: Some(compiler_output), artifacts: cache, ignored_error_codes }
+    }
+
     /// Get the (merged) solc compiler output
     /// ```no_run
     /// use std::collections::BTreeMap;
@@ -637,7 +701,6 @@ impl<T: ArtifactOutput> fmt::Display for ProjectCompileOutput<T> {
 
 #[cfg(test)]
 mod tests {
-
     #[test]
     #[cfg(all(feature = "svm", feature = "async"))]
     fn test_build_all_versions() {
